import os
import logging
import socket
import inspect
import traceback
import uuid
from time import time
from typing import Optional, Any, Union
from colorama import Fore, Style, init
from datetime import datetime
from pvmlib.logs.models import Application, Measurement, DataLogger, ExceptionModel
from pvmlib.logs.utils import LogType, LogLevelColors
from pvmlib.context.request_context import RequestContext
from pvmlib.patterns.decorator import sensitive_info_decorator

DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

logging.basicConfig(format='%(levelname)s: %(asctime)s - %(message)s', datefmt=DATE_FORMAT)
logger = logging.getLogger("uvicorn")

init(autoreset=True)
class LogData:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(LogData, cls).__new__(cls)
            cls._instance.__initialize(*args, **kwargs)
        return cls._instance

    def __initialize(self, origin: str = "INTERNAL"):
        self.logger = logger
        self.schema_version = os.getenv("VERSION_LOG", "1.0.0")
        self.log_origin = origin
        self.tracing_id = "N/A"
        self.hostname = socket.gethostname()
        self.appname = os.getenv("APP_NAME", )
        self.source_ip = socket.gethostbyname(socket.gethostname())
        self.destination_ip = "N/A"
        self.additional_info = {}
        self.app = Application(
            name=os.getenv("APP_NAME", "default"),
            version=os.getenv("API_VERSION", "default"),
            env=os.getenv("ENV", "default"),
            kind=os.getenv("APP_KIND", "default"))
        self.initialized = True

    @sensitive_info_decorator
    def log(
            self, 
            level: int, 
            message: str, 
            logType: str = LogType.INTERNAL, 
            event_type: str = "EVENT", 
            status: str = "SUCCESS", 
            destination_ip: str = None,
            additional_info: Optional[dict] = None,
            exc_info: Union[None, Exception, tuple] = None,
            caller_self: Optional[Any] = None
        ) -> None:
        context = RequestContext()
        
        if destination_ip is not None:
            self.destination_ip = destination_ip
        
        if caller_self is not None:
            method_name = caller_self.__class__.__name__
        else:
            caller_frame = inspect.stack()[2]
            method_name = caller_frame.function

        exception_model = None
        source_file = "N/A"

        if exc_info:
            if isinstance(exc_info, tuple) and len(exc_info) == 3:
                exc_type, exc_value, exc_tb = exc_info
            elif isinstance(exc_info, Exception):
                exc_type, exc_value, exc_tb = type(exc_info), exc_info, exc_info.__traceback__
            else:
                exc_type, exc_value, exc_tb = None, None, None

            if exc_type and exc_value and exc_tb:
                stack_trace = traceback.format_exception(exc_type, exc_value, exc_tb)
                exception_model = ExceptionModel(
                    name=exc_type.__name__,
                    message=str(exc_value),
                    stackTrace=' '.join(stack_trace)
                )
                if additional_info is None:
                    additional_info = {}

                tb = traceback.extract_tb(exc_tb)
                if tb:
                    source_file_trace = tb[-1].filename
                source_file = source_file_trace
        
        log_entry = DataLogger(
            level=logging.getLevelName(level),
            schemaVersion=self.schema_version,
            logType=logType,
            sourceIP=self.source_ip,
            status=status,
            message=message,
            logOrigin=self.log_origin,
            timestamp=datetime.now().strftime(DATE_FORMAT),
            tracingId=context.get_tracing_id(),
            hostname=self.hostname,
            eventType=f"{logType}_{event_type.upper()}",
            application=self.app,
            measurement=Measurement(
                method=method_name,
                elapsedTime=time() - context.get_start_time() if context.get_start_time() else 0,
                status=status,
            ),
            destinationIP=self.destination_ip,
            additionalInfo=additional_info or self.additional_info,
            exception=exception_model,
            sourceFile=source_file
        )

        colored_date = f"{Fore.CYAN}{log_entry.timestamp}{Style.RESET_ALL}"
        colored_name = f"{Fore.CYAN}{self.appname}{Style.RESET_ALL}"
        color_status = getattr(LogLevelColors, log_entry.level, Fore.WHITE) + log_entry.status + Style.RESET_ALL

        log_message = f'{colored_date} - {colored_name} - {log_entry.model_dump()} - Status: {color_status}'
        self.logger.log(level, log_message)

    def info(
            self, 
            message: str, 
            logType: str = LogType.SYSTEM, 
            event_type: str = "EVENT", 
            status: str = "SUCCESS", 
            destination_ip: str = None,
            additional_info: dict = None,
            caller_self: Optional[Any] = None
        ):
        self.log(
            logging.INFO, 
            message, 
            logType, 
            event_type, 
            status, 
            destination_ip,
            additional_info,
            caller_self
        )

    def error(
            self, 
            message: str, 
            logType: str = LogType.SYSTEM, 
            event_type: str = "EVENT", 
            status: str = "ERROR", 
            destination_ip: str = None,
            additional_info: dict = None,
            exc_info: Optional[Exception] = None,
            caller_self: Optional[Any] = None
        ):
        self.log(
            logging.ERROR, 
            message, 
            logType, 
            event_type, 
            status, 
            destination_ip,
            additional_info,
            exc_info,
            caller_self
        )

    def warning(
            self, 
            message: str, 
            logType: str = LogType.SYSTEM, 
            event_type: str = "EVENT", 
            status: str = "WARNING", 
            destination_ip: str = None,
            additional_info: dict = None,
            exc_info: Optional[Exception] = None,
            caller_self: Optional[Any] = None
            ):
        self.log(
            logging.WARNING, 
            message, 
            logType, 
            event_type, 
            status, 
            destination_ip,
            additional_info,
            exc_info,
            caller_self
            )

    def debug(
            self, 
            message: str, 
            logType: str = LogType.SYSTEM, 
            event_type: str = "EVENT", 
            status: str = "DEBUG", 
            destination_ip: str = None,
            additional_info: dict = None,
            exc_info: Optional[Exception] = None,
            caller_self: Optional[Any] = None
            ):
        self.log(
            logging.DEBUG, 
            message, 
            logType, 
            event_type, 
            status, 
            destination_ip, 
            additional_info,
            exc_info,
            caller_self
            )

    def critical(
            self, 
            message: str, 
            logType: str = LogType.SYSTEM, 
            event_type: str = "EVENT", 
            status: str = "CRITICAL", 
            destination_ip: str = None,
            additional_info: dict = None,
            exc_info: Optional[Exception] = None,
            caller_self: Optional[Any] = None
            ):
        self.log(
            logging.CRITICAL, 
            message, 
            logType, 
            event_type, 
            status, 
            destination_ip,
            additional_info,
            exc_info,
            caller_self
            )

class LoggerSingleton:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(LoggerSingleton, cls).__new__(cls)
            cls._instance.__initialize(*args, **kwargs)
            cls._instance._id = str(uuid.uuid4())
        return cls._instance
            
    def __initialize(self, *args, **kwargs):
        self.logger = LogData()
    
    def get_instance_id(self):
        return self._id


import threading

class RequestContext:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super(RequestContext, cls).__new__(cls)
                    cls._instance.reset()
        return cls._instance

    def reset(self):
        self.start_time = None
        self.tracing_id = None
        self.user_id = None
        self.session_id = None
        self.client_ip = None
        self.user_agent = None
        self.request_path = None

    def set_start_time(self, start_time):
        self.start_time = start_time

    def get_start_time(self):
        return self.start_time

    def set_tracing_id(self, tracing_id):
        self.tracing_id = tracing_id

    def get_tracing_id(self):
        return self.tracing_id

    def set_user_id(self, user_id):
        self.user_id = user_id

    def get_user_id(self):
        return self.user_id

    def set_session_id(self, session_id):
        self.session_id = session_id

    def get_session_id(self):
        return self.session_id

    def set_client_ip(self, client_ip):
        self.client_ip = client_ip

    def get_client_ip(self):
        return self.client_ip

    def set_user_agent(self, user_agent):
        self.user_agent = user_agent

    def get_user_agent(self):
        return self.user_agent

    def set_request_path(self, request_path):
        self.request_path = request_path

    def get_request_path(self):
        return self.request_path


from fastapi import HTTPException
from pvmlib.schemas.errors_schema import ErrorGeneralSchema
from pvmlib.logs.logger import LoggerSingleton
from pvmlib.context import RequestContext
from pvmlib.schemas import ResponseMetaSchema
import json
from datetime import datetime
import time

log = LoggerSingleton()

class TypeError:
    errors = {
        500: "INTERNAL_SERVER_ERROR",
        400: "BAD_REQUEST",
        401: "UNAUTHORIZED",
        403: "FORBIDDEN",
        404: "NOT_FOUND",
        405: "METHOD_NOT_ALLOWED",
        409: "CONFLICT",
        415: "UNSUPPORTED_MEDIA_TYPE",
        422: "UNPROCESSABLE_ENTITY",
        501: "NOT_IMPLEMENTED",
        503: "SERVICE_UNAVAILABLE",
        504: "GATEWAY_TIMEOUT"
    }

class ErrorResponseException(HTTPException):
    def __init__(
            self,
            message: str = "ERROR",
            status_code: int = 500,
        ):
        context = RequestContext()
        start_time = context.get_start_time()
        time_elapsed = time.time() - start_time if start_time else 0.0
        transaction_id = context.get_tracing_id()

        response = ErrorGeneralSchema(
            data=None,
            meta=ResponseMetaSchema(
                transactionID=transaction_id,
                status=TypeError.errors.get(status_code, "UNKNOWN_ERROR"),
                statusCode=status_code,
                timestamp=datetime.now().isoformat(),
                message=message,
                time_elapsed=time_elapsed
            ).model_dump()
        )

        super().__init__(status_code=status_code, detail=json.dumps(response.model_dump()))


from typing import Any, Dict, Optional
from pydantic import BaseModel

class Application(BaseModel):
    name: str
    version: str
    env: str
    kind: str

class Measurement(BaseModel):
    method: str
    elapsedTime: float

class ExceptionModel(BaseModel):
    name: str
    message: str
    stackTrace: str

class DataLogger(BaseModel):
    level: str
    schemaVersion: str
    logType: str
    sourceIP: str
    status: str
    message: str
    logOrigin: str
    timestamp: str
    tracingId: str
    hostname: str
    eventType: str
    application: Application
    measurement: Measurement
    destinationIP: str
    additionalInfo: Optional[Dict[str, Any]] = None
    exception: Optional[ExceptionModel] = None
    sourceFile: str = None



import abc
import os
from motor.motor_asyncio import AsyncIOMotorClient
from contextlib import contextmanager
from pvmlib.logs.logger import LoggerSingleton

class Session:
    pass

class Database:
    @abc.abstractmethod
    @contextmanager
    def session(self): pass

class DatabaseManager(Database):
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(DatabaseManager, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        if not hasattr(self, 'initialized'):
            self.mongo_database = None
            self.mongo_client = None
            self.initialized = True

    async def connect_to_mongo(self):
        logger = LoggerSingleton().logger
        try:
            mongo_uri = os.getenv("MONGO_URI", "localhost:27017")
            mongo_timeout_ms = os.getenv("MONGO_TIMEOUT_MS", "5000")
            mongo_max_pool_size = os.getenv("MONGO_MAX_POOL_SIZE", "100")
            mongo_db_name = os.getenv("MONGO_DB_NAME")

            if not mongo_uri or not mongo_db_name:
                raise ValueError("MONGO_URI and MONGO_DB_NAME must be set in environment variables")

            self.mongo_client = AsyncIOMotorClient(
                mongo_uri,
                serverSelectionTimeoutMS=int(mongo_timeout_ms),
                maxPoolSize=int(mongo_max_pool_size)
            )
            self.mongo_database = self.mongo_client[mongo_db_name]
        except Exception as e:
            raise

    async def disconnect_from_mongo(self):
        if self.mongo_client:
            self.mongo_client.close()

    @contextmanager
    def session(self):
        try:
            yield self.mongo_database
        finally:
            pass 

database_manager = DatabaseManager()


from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from pvmlib.context import RequestContext
import time
import uuid

class TracingMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        start_time = time.time()
        tracing_id = request.headers.get("transaction_id", str(uuid.uuid4()))
        user_id = request.headers.get("user_id", "anonymous")
        session_id = request.headers.get("session_id", str(uuid.uuid4()))
        client_ip = request.client.host
        user_agent = request.headers.get("user-agent", "unknown")
        request_path = request.url.path
        context = RequestContext()
        context.set_start_time(start_time)
        context.set_tracing_id(tracing_id)
        context.set_user_id(user_id)
        context.set_session_id(session_id)
        context.set_client_ip(client_ip)
        context.set_user_agent(user_agent)
        context.set_request_path(request_path)
        response = await call_next(request)
        return response


from .utils import Utils
from .dependecy_check import DependencyChecker, check_mongo, check_external_service

__all__ = [
    "Utils",
    "DependencyChecker",
    "check_mongo",
    "check_external_service"
]



import os
import json
import functools
from fastapi import Request
from fastapi.responses import JSONResponse
from pvmlib.fork.actions import Actions
from aiohttp import ClientSession

class MsManager:
    @staticmethod
    def Forks(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)
        return wrapper

    @staticmethod
    def Create(configs, ctx=[]):
        def decorator(func):
            @functools.wraps(func)
            async def wrapper(request: Request, data, *args, **kwargs):
                
                data = {}
                response = {}
                headers = {}

                for key, value in request.headers.items():
                    headers.pop("host", None)
                    headers.update({key: value})

                body_bytes = await request.body()
                if body_bytes:
                    body_json = json.loads(body_bytes)
                    data.update(body_json)
                
                if request.query_params:
                    data.update(request.query_params)

                async with ClientSession() as session:
                    for config in configs:
                        fork_conf = func.__globals__['confFork']().get(config['fork'])
                        if fork_conf:
                            app = fork_conf['App']
                            endpoint = fork_conf['EndPoint']
                            version = fork_conf['Version']
                            url_base = os.environ.get('URL_ENDPOINT_SERVICES', None)
                            url = f"{url_base}/{app}/api/{version}/{endpoint}"
                            query = fork_conf.get('Query', None)
                            
                            action_map = {
                                Actions.POST: lambda: session.post(url, json=data, headers=headers),
                                Actions.GET: lambda: session.get(format_query_params(data, query, url), headers=headers),
                                Actions.UPDATE: lambda: session.put(url, json=data, headers=headers),
                                Actions.DELETE: lambda: session.delete(url, headers=headers)
                            }
                            
                            action = action_map.get(config['action'])
                            if action is None:
                                continue

                            async with action() as response:
                                if response is None:
                                    continue

                                data_response = await response.json()

                                if response.status != 200:
                                    return JSONResponse(status_code=response.status, content=data_response)

                                fnc_name = config['fnc']
                                if fnc_name in func.__globals__:
                                    fnc = func.__globals__[fnc_name]
                                    data = fnc(data, data_response)
                                
                                data.update(data_response['data'])

                return await func(request, data, *args, **kwargs)
            return wrapper
        return decorator

def format_query_params(data, keys, url):
    query_params = []
    for key in keys.split(","):
        if key in data:
            query_params.append(f"{key}={data[key]}")
    
    if query_params:
        url += "?" + "&".join(query_params)
    return url


from pvmlib.schemas.success_schema import ResponseGeneralSchema, ResponseMetaSchema
from typing import Any
from pvmlib.context.request_context import RequestContext
from time import time
from datetime import datetime

class TypeSuccess:
    success = {
        200: "SUCCESS",
        201: "CREATED",
        202: "ACCEPTED",
        204: "NO_CONTENT",
        206: "PARTIAL_CONTENT",
        207: "MULTI_STATUS",
        208: "ALREADY_REPORTED",
        226: "IM_USED"
    }

class SuccessResponse(ResponseGeneralSchema):
    
    def __init__(
            self,
            status_code: int = 200,
            data: Any = {},
            message: str = "Request processed successfully"
        ):
        context = RequestContext()
        start_time = context.get_start_time()
        time_elapsed = time() - start_time if start_time else 0.0
        transaction_id = context.get_tracing_id()

        super().__init__( 
            data=data, 
            meta=ResponseMetaSchema(
                transactionID=transaction_id, 
                status=TypeSuccess.success.get(status_code), 
                statusCode=status_code,
                timestamp=datetime.now().isoformat(),
                message=message,
                time_elapsed=time_elapsed
            )
        )


from pydantic import BaseModel
from typing import Any, Dict
from .response_meta_schema import ResponseMetaSchema

class ResponseGeneralSchema(BaseModel):
    meta: ResponseMetaSchema
    data: Any

type_content = "application/json"

success_general_schema = {
    "definitions": {
        "ResponseGeneralSchema": {
            "type": "object",
            "properties": {
                "meta": {
                    "type": "object",
                    "properties": {
                        "transactionID": {"type": "string"},
                        "status": {"type": "string"},
                        "statusCode": {"type": "integer"},
                        "timestamp": {"type": "string"},
                        "message": {"type": "string"},
                        "time_elapsed": {"type": "float"}
                    }
                },
                "data": {"type": "any"},
            }
        }
    }
}

success_response_example = {
    "description": "Successful Response",
    "content": {
        type_content: {
            "schema": success_general_schema["definitions"]["ResponseGeneralSchema"],
            "examples": {
                "SuccessExample": {
                    "value": ResponseGeneralSchema(
                        meta=ResponseMetaSchema(
                            transactionID="bdae4708-72fd-49a8-8f73-646605101072",
                            status="SUCCESS",
                            statusCode=200,
                            timestamp="2025-02-19T00:00:00Z",
                            message="Success",
                            time_elapsed=0.0
                        ),
                        data={"dns": "https://example.com/puntodeventa/api/v1/"},
                    ).model_dump()
                }
            }
        }
    }
}


from pydantic import BaseModel
from typing import Any
from .response_meta_schema import ResponseMetaSchema

class ErrorGeneralSchema(BaseModel):
    meta: ResponseMetaSchema
    data: Any

type_content = "application/json"

error_general_schema = {
    "definitions": {
        "ErrorGeneralSchema": {
            "type": "object",
            "properties": {
                "meta": {
                    "type": "object",
                    "properties": {
                        "transactionID": {"type": "string"},
                        "status": {"type": "string"},
                        "statusCode": {"type": "integer"},
                        "timestamp": {"type": "string"},
                        "message": {"type": "string"},
                        "time_elapsed": {"type":"float"} 
                    }
                },
                "data": {"type": "any"},
            }
        }
    }
}

error_response_general_405 = {
    "description": "Method Not Allowed",
    "content": {
        type_content: {
            "schema": error_general_schema["definitions"]["ErrorGeneralSchema"],
            "examples": {
                "MethodNotAllowed": {
                    "value": ErrorGeneralSchema(
                        data={"user_message": "Method Not Allowed"},
                        meta=ResponseMetaSchema(
                            transactionID="unknown",
                            status="error",
                            statusCode=405,
                            timestamp="2025-02-19T00:00:00Z",
                            message="Method Not Allowed",
                            time_elapsed=0.0
                        )
                    ).model_dump()
                }
            }
        }
    }
}


from fastapi import FastAPI, Request, HTTPException
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from starlette.status import (
    HTTP_422_UNPROCESSABLE_ENTITY,
    HTTP_405_METHOD_NOT_ALLOWED,
    HTTP_500_INTERNAL_SERVER_ERROR,
    HTTP_400_BAD_REQUEST,
    HTTP_404_NOT_FOUND
)
from pvmlib.responses.error_response import ErrorResponseException
from pvmlib.utils import Utils
from pvmlib.logs import LoggerSingleton, LogType
import json

EVENT_TYPES = {
    HTTP_500_INTERNAL_SERVER_ERROR: "SERVER_ERROR",
    HTTP_422_UNPROCESSABLE_ENTITY: "VALIDATION_ERROR",
    HTTP_404_NOT_FOUND: "NOT_FOUND",
    HTTP_405_METHOD_NOT_ALLOWED: "METHOD_NOT_ALLOWED",
    HTTP_400_BAD_REQUEST: "BAD_REQUEST",
    ErrorResponseException: "ERROR_RESPONSE"
}

class ExceptionHandlers:
    def __init__(self):
        self.log = LoggerSingleton().logger

    async def internal_server_error_exception_handler(self, request: Request, exc: Exception):
        error_message, error_info = await Utils.get_instance_exception(exc)
        response = ErrorResponseException(
            message=error_message,
            status_code=HTTP_500_INTERNAL_SERVER_ERROR
        )

        self.log.error(
            message=error_info,
            exc_info=exc,
            caller_self=self,
            logType=LogType.INTERNAL,
            event_type=EVENT_TYPES[HTTP_500_INTERNAL_SERVER_ERROR],
            status=str(HTTP_500_INTERNAL_SERVER_ERROR)
        )

        return JSONResponse(content=json.loads(response.detail), status_code=HTTP_500_INTERNAL_SERVER_ERROR)

    async def validation_exception_handler(self, request: Request, exc: RequestValidationError): 
        error_details = Utils.get_error_details(exc.errors())
        error_message = "Validation error: " + ", ".join(error_details)
        response = ErrorResponseException(
            message=error_message,
            status_code=HTTP_422_UNPROCESSABLE_ENTITY
        )

        additional_info = {
            "url": str(request.url),
            "endpoint": request.scope.get("endpoint").__name__ if request.scope.get("endpoint") else None
        }

        self.log.warning(
            message=error_message,
            exc_info=exc,
            caller_self=self,
            logType=LogType.SYSTEM,
            event_type=EVENT_TYPES[HTTP_422_UNPROCESSABLE_ENTITY],
            destination_ip=request.client.host,
            additional_info=additional_info,
            status=str(HTTP_422_UNPROCESSABLE_ENTITY)
        )

        return JSONResponse(content=json.loads(response.detail), status_code=HTTP_422_UNPROCESSABLE_ENTITY)

    async def not_found_exception_handler(self, request: Request, exc: HTTPException):
        error_message = "Resource not found"
        response = ErrorResponseException(
            message=error_message,
            status_code=HTTP_404_NOT_FOUND
        )

        additional_info = {
            "url": str(request.url),
            "endpoint": request.scope.get("endpoint").__name__ if request.scope.get("endpoint") else None
        }

        self.log.warning(
            message=error_message,
            exc_info=exc,
            caller_self=self,
            logType=LogType.SYSTEM,
            event_type=EVENT_TYPES[HTTP_404_NOT_FOUND],
            destination_ip=request.client.host,
            additional_info=additional_info,
            status=str(exc.status_code)
        )

        return JSONResponse(content=json.loads(response.detail), status_code=exc.status_code) 

    async def method_not_allowed_exception_handler(self, request: Request, exc: HTTPException):   
        error_message = "Method not allowed."
        response = ErrorResponseException(
            message=error_message,
            status_code=HTTP_405_METHOD_NOT_ALLOWED
        )

        self.log.warning(
            message=error_message,
            exc_info=exc,
            caller_self=self,
            logType=LogType.SYSTEM,
            event_type=EVENT_TYPES[HTTP_405_METHOD_NOT_ALLOWED],
            status=str(exc.status_code)
        )

        return JSONResponse(content=json.loads(response.detail), status_code=exc.status_code)

    async def bad_request_exception_handler(self, request: Request, exc: HTTPException):
        error_message = "Bad request."
        response = ErrorResponseException(
            message=error_message,
            status_code=HTTP_400_BAD_REQUEST
        )

        self.log.warning(
            message=error_message,
            exc_info=exc,
            caller_self=self,
            logType=LogType.SYSTEM,
            event_type=EVENT_TYPES[HTTP_400_BAD_REQUEST],
            status=str(exc.status_code)
        )

        return JSONResponse(content=json.loads(response.detail), status_code=exc.status_code)

    async def error_exception_handler(self, request: Request, exc: ErrorResponseException):
        self.log.error(
            message=exc.message,
            exc_info=exc,
            caller_self=self,
            logType=LogType.SYSTEM,
            event_type=EVENT_TYPES[ErrorResponseException],
        )
        return JSONResponse(content=json.loads(exc.detail), status_code=exc.status_code)

def register_exception_handlers(app: FastAPI):
    handlers = ExceptionHandlers()
    app.add_exception_handler(HTTP_500_INTERNAL_SERVER_ERROR, handlers.internal_server_error_exception_handler)
    app.add_exception_handler(RequestValidationError, handlers.validation_exception_handler)
    app.add_exception_handler(HTTP_404_NOT_FOUND, handlers.not_found_exception_handler)
    app.add_exception_handler(HTTP_405_METHOD_NOT_ALLOWED, handlers.method_not_allowed_exception_handler)
    app.add_exception_handler(HTTP_400_BAD_REQUEST, handlers.bad_request_exception_handler)
    app.add_exception_handler(ErrorResponseException, handlers.error_exception_handler)


from colorama import Fore

class LogType:
    SYSTEM = "SYSTEM"
    TRANSACTION = "TRANSACTION"
    SECURITY = "SECURITY"
    SUCCESS = "SUCCESS"
    FAILED = "FAILED"
    INTERNAL = "INTERNAL"
    DEPENDENCY = "DEPENDENCY"


class LogLevelColors:
    DEBUG = Fore.BLUE
    INFO = Fore.GREEN
    WARNING = Fore.YELLOW
    ERROR = Fore.RED
    CRITICAL = Fore.RED


from contextlib import asynccontextmanager
from fastapi import FastAPI
from .database import database_manager

@asynccontextmanager
async def lifespan(app: FastAPI):
    try:
        await database_manager.connect_to_mongo()
    except Exception as e:
        raise e

    yield
    await database_manager.disconnect_from_mongo()


from fastapi import APIRouter
from pvmlib.schemas.healthcheck_reponse_schema import responses_liveness, LivenessResponse

liveness_router = APIRouter()

@liveness_router.get(
    "/healthcheck/liveness", 
    tags=["Health Check"], 
    responses=responses_liveness, 
    summary="Status del servicio",
    response_model=LivenessResponse
)
async def liveness() -> LivenessResponse:
    """ Comprueba que el servicio este operando """
    return LivenessResponse(
        status="UP",
        code=200,
        dependencies={}
    )


import os
from fastapi import APIRouter
from pvmlib.schemas.healthcheck_reponse_schema import responses_readiness, ReadinessResponse
from pvmlib.utils.dependecy_check import DependencyChecker, check_mongo, check_external_service
from pvmlib.database.database import DatabaseManager

readiness_router = APIRouter()
database_manager = DatabaseManager()

dependency_services = os.getenv("SERVICES_DEPENDENCY_HEALTHCHECK", "").split(",") 

@readiness_router.get(
    "/healthcheck/readness", 
    tags=["Health Check"], 
    responses=responses_readiness, 
    summary="Status de los recursos",
    response_model=ReadinessResponse
)
async def readiness() -> ReadinessResponse:
    """ Comprueba que el servicio este operando """
    dependencies = []

    if dependency_services and any(dependency_services):
        dependencies.extend([lambda url=url: check_external_service(url) for url in dependency_services if url])

    if database_manager.mongo_client:
        dependencies.append(lambda: check_mongo(database_manager))

    if not dependencies:
        return ReadinessResponse(
            status="ready",
            code=200,
            dependencies={}
        )

    checker = DependencyChecker(dependencies=dependencies)
    dependencies_status = await checker.check_dependencies()

    return ReadinessResponse(
        status="ready" if all(status == "UP" for status in dependencies_status.values()) else "not ready",
        code=200 if all(status == "UP" for status in dependencies_status.values()) else 500,
        dependencies=dependencies_status
    )


error_messages = {
    "ConnectionFailure": "Connection failure occurred.",
    "OperationFailure": "Operation failure occurred.",
    "ServerSelectionTimeoutError": "Server selection timeout error occurred.",
    "CollectionInvalid": "Collection invalid error occurred.",
    "ConfigurationError": "Configuration error occurred.",
    "DuplicateKeyError": "Duplicate key error occurred.",
    "InvalidURI": "Invalid URI error occurred.",
    "WriteError": "Write error occurred.",
    "WriteConcernError": "Write concern error occurred.",
    "NetworkTimeout": "Network timeout error occurred.",
    "InvalidName": "Invalid name error occurred.",
    "InvalidOperation": "Invalid operation error occurred.",
    "ConnectionError": "Connection error occurred.",
    "RetryError": "Retry error occurred.",
    "TimeoutError": "Timeout error occurred.",
    "ValueError": "Value error occurred.",
    "TypeError": "Type error occurred.",
    "KeyError": "Key error occurred.",
    "IndexError": "Index error occurred.",
    "AttributeError": "Attribute error occurred.",
    "JSONDecodeError": "JSON decode error occurred.",
    "DatabaseError": "Database error occurred.",
    "IntegrityError": "Integrity error occurred.",
    "OperationalError": "Operational error occurred.",
    "ProgrammingError": "Programming error occurred.",
    "DataError": "Data error occurred.",
    "NotSupportedError": "Not supported error occurred.",
    "FileNotFoundError": "File not found error occurred.",
    "PermissionError": "Permission error occurred.",
    "IsADirectoryError": "Is a directory error occurred.",
    "NotADirectoryError": "Not a directory error occurred.",
    "OSError": "OS error occurred.",
    "ZeroDivisionError": "Division by zero error occurred.",
    "ImportError": "Import error occurred.",
    "ModuleNotFoundError": "Module not found error occurred.",
    "MemoryError": "Memory error occurred.",
    "OverflowError": "Overflow error occurred.",
    "RecursionError": "Recursion error occurred.",
    "ReferenceError": "Reference error occurred.",
    "RuntimeError": "Runtime error occurred.",
    "SyntaxError": "Syntax error occurred.",
    "IndentationError": "Indentation error occurred.",
    "TabError": "Tab error occurred.",
    "SystemError": "System error occurred.",
    "SystemExit": "System exit occurred.",
    "KeyboardInterrupt": "Keyboard interrupt occurred.",
    "GeneratorExit": "Generator exit occurred.",
    "StopIteration": "Stop iteration occurred.",
    "StopAsyncIteration": "Stop async iteration occurred."
}


from pvmlib.database import DatabaseManager, lifespan
from pvmlib.middleware.request_middleware import TracingMiddleware
from pvmlib.healthchecks import liveness_router, readiness_router
from pvmlib.patterns import circuit_breaker, sensitive_info_decorator
from pvmlib.fork import Actions, MsManager
from pvmlib.logs import LoggerSingleton, Application, Measurement, LogType
from pvmlib.exceptions import (
    RequestValidationError,
    register_exception_handlers
)

from pvmlib.responses import SuccessResponse, ErrorResponseException

from pvmlib.schemas import (
    ResponseMetaSchema,
    success_general_schema,
    ErrorGeneralSchema,
    error_general_schema,
)

name = 'pvmlib'

__all__ = [
    #middleware
    "TracingMiddleware",
    #database connection
    "DatabaseManager",
    "lifespan",
    #healthchecks
    "liveness_router",
    "readiness_router",
    #patterns
    "circuit_breaker",
    "sensitive_info_decorator",
    #fork
    "MsManager",
    "Actions",
    #logs
    "LoggerSingleton",
    "Application",
    "Measurement",
    "LogType",
    #responses
    "SuccessResponse",
    "ErrorResponseException",
    #exceptions
    "RequestValidationError",
    "register_exception_handlers",
    #schemas
    "ResponseGeneralSchema",
    "ResponseMetaSchema",
    "success_general_schema",
    "ErrorGeneralSchema",
    "error_general_schema",
]


from .error_message_default import error_messages

class Utils:
    @staticmethod
    def get_method_name(obj, func_name: str = '') -> str:
        obj_class_name = f"{obj.__class__.__module__}.{obj.__class__.__qualname__}"
        full_name = obj_class_name + '.' + func_name if func_name else obj_class_name
        return full_name
    
    @staticmethod
    def add_attributes(obj, data: dict) -> None:
        for key, value in data.items():
            setattr(obj, key, value)
    
    @staticmethod
    def discard_empty_attributes(obj) -> None:
        obj_copy = obj.__dict__.copy()
        for key, value in obj_copy.items():
            if not value:
                delattr(obj, key)
    
    @staticmethod
    def sort_attributes(obj) -> None:
        obj.__dict__ = dict(sorted(obj.__dict__.items()))
    
    @staticmethod
    def get_error_details(errors: list) -> list:
        return list(map(lambda error: f"Falta parámetro '{error['loc'][1]}'" if error['type'] == 'missing' else f"El parámetro '{error['loc'][1]}' debe ser de tipo '{error['type']}'", errors))
    
    @staticmethod 
    async def get_instance_exception(exc: Exception):
        exception_class_name = type(exc).__name__
        error_message = error_messages.get(exception_class_name, "An unexpected error occurred.")
        error_info = str(exc)
        return error_message, error_info


from .errors_schema import (
    ErrorGeneralSchema,
    error_general_schema,
)
from .success_schema import (
    ResponseGeneralSchema, 
    success_general_schema,
)

from .response_meta_schema import ResponseMetaSchema

__all__ = [
    # Success
    "ResponseGeneralSchema",
    "success_general_schema",
    # Errors
    "ErrorGeneralSchema",
    "error_general_schema",
    # Meta
    "ResponseMetaSchema",
]


from typing import List, Dict, Callable
from fastapi import HTTPException
from pvmlib.database.database import DatabaseManager
from aiohttp import ClientSession
from urllib.parse import urlparse

class DependencyChecker:
    def __init__(self, dependencies: List[Callable[[], Dict[str, str]]]):
        self.dependencies = dependencies

    async def check_dependencies(self) -> Dict[str, str]:
        results = {}
        for check in self.dependencies:
            try:
                result = await check()
                results.update(result)
            except Exception as e:
                raise HTTPException(status_code=500, detail=f"Dependency check failed: {str(e)}")
        return results

async def check_mongo(database_manager: DatabaseManager) -> Dict[str, str]:
    try:
        # Verificar si ya hay una conexión existente
        if not database_manager.mongo_client:
            await database_manager.connect_to_mongo()
        
        await database_manager.mongo_database.command("ping")
        return {"mongodb": "UP"}
    except Exception as e:
        return {"mongodb": "DOWN"}

async def check_external_service(url: str) -> Dict[str, str]:
    try:
        async with ClientSession() as session:
            async with session.get(url) as response:
                if response.status == 200:
                    return {urlparse(url).hostname: "UP"}
                else:
                    return {urlparse(url).hostname: "DOWN"}
    except Exception as e:
        return {urlparse(url).hostname: "DOWN"}


from pydantic import BaseModel
from typing import Dict

class LivenessResponse(BaseModel):
    status: str
    code: int
    dependencies: Dict[str, str]

class ReadinessResponse(BaseModel):
    status: str
    code: int
    dependencies: Dict[str, str]

responses_liveness = {
    200: {
        "description": "Successful Response",
        "content": {
            "application/json": {
                "schema": {
                    "$ref": "#/components/schemas/LivenessResponse"
                },
                "example": {
                    "status": "UP",
                    "code": 200,
                    "dependencies": {
                        "mongodb": "UP",
                        "auth-service": "UP",
                        "payment-service": "UP",
                        "notification-service": "DOWN"
                    }
                }
            }
        }
    }
}

responses_readiness = {
    200: {
        "description": "Successful Response",
        "content": {
            "application/json": {
                "schema": {
                    "$ref": "#/components/schemas/ReadinessResponse"
                },
                "example": {
                    "status": "ready",
                    "code": 200,
                    "dependencies": {
                        "mongodb": "UP",
                        "auth-service": "UP",
                        "payment-service": "UP",
                        "notification-service": "DOWN"
                    }
                }
            }
        }
    }
}